import sys

from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

from craftslicer.core.mc.palette_list import palette_list


class AssignBlocksDialog(QMessageBox):
    def __init__(self):
        super().__init__()

        self.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
        self.setDefaultButton(QMessageBox.Cancel)

        self.parent_layout = QGridLayout()
        self.parent_layout.addWidget(QLabel("Select Minecraft Version"), 0, 0)
        # self.layout().addWidget(QLabel("Select Minecraft Version"))

        self.version_list = QComboBox()
        self.version_list.addItems(palette_list.keys())
        self.parent_layout.addWidget(self.version_list, 0, 1)
        # self.layout().addWidget(self.version_list)
        self.layout().addLayout(self.parent_layout, 0, 0)

        self.setWindowTitle("Assign Blocks")

        self.setWindowModality(Qt.ApplicationModal)
        self.setWindowFlags(Qt.CustomizeWindowHint | Qt.WindowTitleHint)


class DeployBlocksDialog(QMessageBox):
    def __init__(self):
        super(DeployBlocksDialog, self).__init__()

        self.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
        self.setDefaultButton(QMessageBox.Cancel)

        self.parent_layout = QGridLayout()
        self.parent_layout.addWidget(QLabel("Author"), 0, 0)
        self.author_text = QLineEdit()
        self.parent_layout.addWidget(self.author_text, 0, 1)

        self.parent_layout.addWidget(QLabel("Project Name"), 1, 0)
        self.project_text = QLineEdit()
        self.parent_layout.addWidget(self.project_text, 1, 1)

        self.parent_layout.setColumnStretch(0, 1)
        self.parent_layout.setColumnStretch(1, 3)

        self.layout().addLayout(self.parent_layout, 0, 0)

        self.setWindowTitle("Deploy Blocks")

        self.setWindowModality(Qt.ApplicationModal)
        self.setWindowFlags(Qt.CustomizeWindowHint | Qt.WindowTitleHint)


class MainWindow(QMainWindow):
    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        self.setMinimumSize(128, 128)

        self.show()

        dialog = DeployBlocksDialog()
        ret = dialog.exec_()

        if ret == QMessageBox.Cancel or ret == QMessageBox.Close or ret == QMessageBox.Ok:
            exit()


if __name__ == '__main__':
    app = QApplication(sys.argv)

    w = MainWindow()
    app.exec_()



import os
import time

import tempfile

import numpy as np
from nbt.nbt import NBTFile, \
    TAG_Compound, \
    TAG_Long_Array, \
    TAG_List, \
    TAG_Int, \
    TAG_Long, \
    TAG_String, \
    TAG_BYTE, \
    TAG_COMPOUND
from trimesh import remesh

from craftslicer.core.gl.gl_elements import GlModel, GlGrid, GlVoxel, GlImage
from craftslicer.core.gl.gl_processor import position_matrix
from craftslicer.core.net.socket import send


def convert_voxels(models: list[GlModel], grid: GlGrid):
    maxes = np.array(grid.grid_maxes)
    voxels = np.zeros((maxes + 1).astype(int), dtype=int)

    for model in models:
        vertices = np.array(model.face_vertices).reshape((-1, 7))
        transformation_matrix = position_matrix(model.theta, model.position, model.scale, reverse=True)

        vertices = np.delete(vertices, [3, 4, 5], axis=1)
        vertices = vertices @ transformation_matrix
        vertices = np.delete(vertices, 3, axis=1)

        vertices = np.add(vertices, maxes / 2)
        faces = np.array(model.indices).reshape((-1, 3))

        vertices, faces = remesh.subdivide_to_size(vertices=vertices, faces=faces, max_edge=1., max_iter=32)

        indices = np.unique(vertices.astype(int), axis=0).reshape((-1, 3))
        indices = np.clip(indices, [0, 0, 0], maxes.astype(int))
        voxels[indices[:, 0], indices[:, 1], indices[:, 2]] = 1

    return voxels


def texture_voxels(voxels: GlVoxel, images: list[GlImage]):
    voxels = voxels.voxels

    voxel_color = np.zeros((*voxels.shape, 3), dtype=int)

    depth = 200
    layer_depth = 1
    for image in images:
        pixels = np.array(image.image)
        if pixels.shape[-1] == 4:
            pixels = np.delete(pixels, 3, axis=2)

        maxes_skew = (np.array([*image.maxes, 0]) / 2).astype(int)
        pos_skew = np.array([*image.position, 0]).astype(int)

        pixel_indices = np.array([i for i in np.ndindex(1, *image.size)])
        pixel_indices = np.flip(pixel_indices, axis=1)
        pixel_indices = np.tile(pixel_indices, (depth, 1))

        voxel_indices = np.array([i for i in np.ndindex(depth, *image.size, 1)])
        voxel_indices += -maxes_skew + pos_skew
        voxel_indices = voxel_indices @ np.array(
            position_matrix(image.theta, image.position, image.scale, reverse=True))
        voxel_indices += maxes_skew
        voxel_indices = voxel_indices.astype(int)
        voxel_indices = np.delete(voxel_indices, 3, axis=1)

        a = 0 <= voxel_indices
        a &= voxel_indices < voxels.shape
        a = a[:, 0] & a[:, 1] & a[:, 2]
        voxel_indices = voxel_indices[a]
        pixel_indices = pixel_indices[a]

        b = voxels[voxel_indices[:, 0], voxel_indices[:, 1], voxel_indices[:, 2]] == 1
        voxel_indices = voxel_indices[b]
        pixel_indices = pixel_indices[b]

        pixel_indices_ = pixel_indices.copy()
        c = np.array([], dtype=int)
        for i in range(layer_depth):
            pixel_indices_[c] = -1
            _, d = np.unique(pixel_indices_, axis=0, return_index=True)
            d = d[1:]
            c = np.append(c, d)
        pixel_indices = pixel_indices[c]
        voxel_indices = voxel_indices[c]

        voxel_color[voxel_indices[:, 0], voxel_indices[:, 1], voxel_indices[:, 2]] = \
            pixels[pixel_indices[:, 0], pixel_indices[:, 1]]

    return voxel_color


def assign_blocks(voxels: np.ndarray, voxel_color: np.ndarray, palette: dict):
    voxels = np.repeat(np.expand_dims(voxels, axis=3), 3, axis=3)

    voxel_color = np.where(voxel_color == [0, 0, 0], [126, 126, 126], voxel_color)
    voxel_color *= voxels
    voxel_color = np.reshape(voxel_color, (-1, 3), "F")
    voxel_color_shape = voxel_color.shape

    colors = np.array(list(palette.values()))
    colors_shape = colors.shape
    colors = np.tile(colors, (voxel_color.shape[0], 1))
    voxel_color = np.repeat(voxel_color, colors_shape[0], axis=0)

    colors_diff = np.absolute(voxel_color - colors)
    colors_diff = colors_diff.reshape((voxel_color_shape[0], colors_shape[0], 3))
    colors_diff = np.sum(colors_diff, axis=2)

    flattened_blocks = np.argmin(colors_diff, axis=1)
    block_types = np.array(list(palette.keys()), dtype=str)
    flattened_blocks = block_types[flattened_blocks]

    return flattened_blocks


def deploy_blocks(voxels: np.ndarray, flattened_blocks: np.ndarray, user_info: dict):
    block_state_palette, block_states = np.unique(flattened_blocks, return_inverse=True)

    bit_pack_length = len(np.binary_repr(np.int64(len(block_state_palette) - 1)))
    bit_array_length = int(np.ceil(block_states.shape[0] * bit_pack_length / 64)) * 64

    bit_array = np.zeros(bit_array_length, dtype=bool)
    bit_block_states = block_states[:, None] & (1 << np.arange(bit_pack_length)) > 0
    bit_block_states = bit_block_states.flatten()
    bit_indices = np.arange(0, block_states.shape[0] * bit_pack_length)

    bit_array[bit_indices] = bit_block_states

    bit_array = bit_array.astype(int)
    block_states_long_array = np.packbits(bit_array.reshape((-1, 8, 8))[:, :, ::-1]).view(np.int64)

    nbt_structure = {
        "Metadata": {"class": TAG_Compound, "tags": {
            "EnclosingSize": {"class": TAG_Compound, "tags": {
                "x": {"class": TAG_Int, "value": voxels.shape[0]},
                "y": {"class": TAG_Int, "value": voxels.shape[2]},
                "z": {"class": TAG_Int, "value": voxels.shape[1]}
            }},
            "Author": {"class": TAG_String, "value": user_info["Author"]},
            "Description": {"class": TAG_String, "value": "Generated with CraftSlicer"},
            "Name": {"class": TAG_String, "value": user_info["Name"]},
            "RegionCount": {"class": TAG_Int, "value": 1},
            "TimeCreated": {"class": TAG_Long, "value": int(time.time())},
            "TimeModified": {"class": TAG_Long, "value": int(time.time())},
            "TotalBlocks": {"class": TAG_Int, "value": int(np.unique(voxels, return_counts=True)[1][1])},
            "TotalVolume": {"class": TAG_Int, "value": int(np.prod(voxels.shape))}
        }},
        "Regions": {"class": TAG_Compound, "tags": {
            "Region": {"class": TAG_Compound, "tags": {
                "Position": {"class": TAG_Compound, "tags": {
                    "x": {"class": TAG_Int, "value": 0},
                    "y": {"class": TAG_Int, "value": 0},
                    "z": {"class": TAG_Int, "value": 0}
                }},
                "Size": {"class": TAG_Compound, "tags": {
                    "x": {"class": TAG_Int, "value": voxels.shape[0]},
                    "y": {"class": TAG_Int, "value": voxels.shape[2]},
                    "z": {"class": TAG_Int, "value": voxels.shape[1]}
                }},
                "BlockStatePalette": {"class": TAG_List, "tagID": TAG_COMPOUND, "tags": {
                    "".join(" " for _ in range(i)):
                        {"class": TAG_Compound, "tags": {
                            "Name": {"class": TAG_String, "value": block_state}
                        }}
                    for i, block_state in enumerate(block_state_palette)
                }},
                "Entities": {"class": TAG_List, "tagID": TAG_BYTE, "tags": {}},
                "PendingBlockTicks": {"class": TAG_List, "tagID": TAG_BYTE, "tags": {}},
                "PendingFluidTicks": {"class": TAG_List, "tagID": TAG_BYTE, "tags": {}},
                "TileEntities": {"class": TAG_List, "tagID": TAG_BYTE, "tags": {}},
                "BlockStates": {"class": TAG_Long_Array, "value": block_states_long_array}
            }}
        }},
        "MinecraftDataVersion": {"class": TAG_Int, "value": 2586},
        "Version": {"class": TAG_Int, "value": 5}
    }

    nbt_file = NBTFile()
    nbt_file.name = user_info["Name"]

    def handle_recursive(parent, items: dict):
        for key, value in items.items():
            tag = value["class"]()

            if " " in key:
                tag.name = ""
            else:
                tag.name = key

            if "tagID" in value:
                tag.tagID = value["tagID"]

            if "tags" in value:
                handle_recursive(tag, value["tags"])
            elif "value" in value:
                tag.value = value["value"]
            parent.tags.append(tag)

    handle_recursive(nbt_file, nbt_structure)

    nbt_file.write_file(path := os.path.join(tempfile.gettempdir(), "%s.litematic" % user_info["Name"]))
    send(path)


if __name__ == '__main__':
    voxels_ = np.load("../tests/voxels.npy")
    # colors_ = np.load("../tests/voxel_color.npy")
    # from craftslicer.core.mc.one_dot_sixteen.one_dot_sixteen import palette as palette_

    # flattened_blocks_ = assign_blocks(voxels_, colors_, palette_)
    # np.save("../tests/flattened_blocks_.npy", flattened_blocks_)

    flattened_blocks_ = np.load("../tests/flattened_blocks_.npy")
    deploy_blocks(voxels_, flattened_blocks_, {
        "Author": "Aquarel",
        "Name": "test"
    })

    # noinspection PyPackageRequirements
    from tests import java_socket

    java_socket.send()



import os

# noinspection PyPackageRequirements
import stl
from PIL import Image

from craftslicer.core.gl.gl_processor import *


class GlElement:
    def __init__(self, widget):
        self.widget = widget

        self.theta = [0, 0, 0]
        self.position = [0, 0, 0]
        self.scale = [1, 1, 1]

        self.face_vertices, self.line_vertices, self.vertices, self.indices = None, None, None, None
        self.vao, self.vbo, self.ebo = None, None, None

        self.transformation_matrix = None
        self.set_transformation_matrix()

        self.visible = True

    def gl_calls(self):
        self.vao, self.vbo, self.ebo = create_vao(
            [self.face_vertices, self.indices],
            [self.line_vertices, self.indices])

    def set_transformation_matrix(self):
        self.transformation_matrix = position_matrix(self.theta, self.position, self.scale, reverse=True) \
                                     @ position_matrix(self.widget.theta, [0., 0., 0.], [1., 1., 1.])

    def draw(self):
        self.set_transformation_matrix()

        if not self.visible:
            return

        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
        glBindVertexArray(self.vao[0])
        glUniformMatrix4fv(self.widget.model_loc, 1, GL_FALSE, self.transformation_matrix)
        glDrawElements(GL_TRIANGLES, len(self.indices), GL_UNSIGNED_INT, None)

        glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
        glBindVertexArray(self.vao[1])
        glUniformMatrix4fv(self.widget.model_loc, 1, GL_FALSE, self.transformation_matrix)
        glLineWidth(1)
        glDrawElements(GL_TRIANGLES, len(self.indices), GL_UNSIGNED_INT, None)

        glBindVertexArray(0)


class GlElementSimple(GlElement):
    def __init__(self, widget):
        super(GlElementSimple, self).__init__(widget)

    def gl_calls(self):
        self.vao, self.vbo, self.ebo = create_vao([self.vertices, self.indices])


class GlModel(GlElement):
    def __init__(self, file, widget):
        super(GlModel, self).__init__(widget)

        self.file = file
        self.filename = os.path.basename(self.file)
        self.model_mesh = stl.mesh.Mesh.from_file(file)

        self.face_vertices, self.line_vertices, self.vertices, self.indices = vector_gl(self.model_mesh.vectors)
        self.maxes = np.amax(self.vertices, axis=0)


class GlVoxel(GlElement):
    def __init__(self, voxel, widget, voxel_color=None):
        super().__init__(widget)

        self.voxels = voxel
        self.voxel_color = voxel_color
        self.flattened_blocks = None

        self.face_vertices, self.line_vertices, self.vertices, self.indices = voxel_gl(voxel,
                                                                                       self.widget.grid_maxes,
                                                                                       voxel_color=voxel_color)


class GlImage(GlElementSimple):
    def __init__(self, file, widget):
        super(GlImage, self).__init__(widget)

        self.file = file
        self.filename = os.path.basename(self.file)
        self.pil_image = Image.open(file)

        self.maxes = [i for i in self.widget.grid_maxes]
        self.vertices, self.indices, self.size, self.image = image_gl(self.pil_image, self.widget.grid_maxes)

    def draw(self):
        self.set_transformation_matrix()

        if not self.visible:
            return

        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
        glBindVertexArray(self.vao)
        glUniformMatrix4fv(self.widget.model_loc, 1, GL_FALSE, self.transformation_matrix)
        glDrawElements(GL_TRIANGLES, len(self.indices), GL_UNSIGNED_INT, None)

        glBindVertexArray(0)


class GlGrid(GlElementSimple):
    def __init__(self, widget, maxes=None):
        super().__init__(widget)

        if maxes:
            self.maxes = maxes
        else:
            self.maxes = np.sum([model.maxes for model in self.widget.models], axis=0)

        self.vertices, self.indices, self.widget.grid_maxes = grid_gl(*self.maxes)
        self.grid_maxes = self.widget.grid_maxes

    def draw(self):
        self.set_transformation_matrix()

        glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
        glBindVertexArray(self.vao)
        glUniformMatrix4fv(self.widget.model_loc, 1, GL_FALSE, self.transformation_matrix)
        glLineWidth(2)
        glDrawElements(GL_LINES, len(self.indices), GL_UNSIGNED_INT, None)

        glBindVertexArray(0)



import sys
import os
import numpy as np
import pyrr.matrix44
# noinspection PyPackageRequirements
from OpenGL.GL import *
from PyQt5.QtCore import Qt, QThreadPool, QMutex
from PyQt5.QtGui import *
from PyQt5.QtOpenGL import *
from PyQt5.QtWidgets import *
# noinspection PyUnresolvedReferences
from craftslicer.core.res import qrc_resources

from craftslicer.core.gl.gl_elements import GlModel, GlImage, GlVoxel, GlGrid
from craftslicer.core.gl.gl_processor import display_setup
from craftslicer.core.qt.qt_threads import get_run_options, ImportRunnable


# from craftslicer.core.qt.qt_threads import \
#     ImportRunnable, \
#     ConvertVoxelsRunnable, \
#     TextureVoxelsRunnable, \
#     AssignBlocksRunnable, \
#     DeployBlocksRunnable


class MainWindow(QMainWindow):
    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        self.setMinimumSize(1280, 720)

        self.gl_widget = GlWidget(self)
        self.gl_widget.setFocusPolicy(Qt.FocusPolicy.StrongFocus)

        parent_layout = QHBoxLayout()
        parent_layout.addWidget(self.gl_widget)

        widget = QWidget()
        widget.setLayout(parent_layout)
        self.setCentralWidget(widget)

        self.show()


class GlWidget(QGLWidget):
    def __init__(self, window, *__args):
        super().__init__(*__args)
        self.window = window

        self.theta = [np.pi / 2, np.pi, 0]
        self.theta_ = [np.pi / 2, np.pi, 0]

        self.translate = [0, 0]
        self.translate_ = [0, 0]

        self.zoom = -6

        self.mouse = [0, 0]
        self.vector = [0, 0]
        self.is_rotate = False
        self.is_translate = False

        self.model_loc, self.proj_loc, self.view_loc, self.shader = None, None, None, None
        self.models: list[GlModel] = []
        self.images: list[GlImage] = []
        self.voxels: list[GlVoxel] = []

        def full_path(rel_path):
            return os.path.join(os.path.dirname(__file__), rel_path)

        self.models.append(GlModel(full_path("../res/3d/statue.stl"), self))
        # self.models.append(GlModel("craftslicer/core/res/3d/statue.stl", self))
        self.grid = GlGrid(self)
        self.images.append(GlImage(full_path("../res/3d/statue.png"), self))
        # self.images.append(GlImage("core/res/3d/statue.png", self))

        self.setAcceptDrops(True)
        self.thread_pool = QThreadPool.globalInstance()

        self.buffer_mutex = QMutex()
        self.buffer_mutex.lock()
        self.model_buffer: list[GlModel] = []
        self.image_buffer: list[GlImage] = []
        self.voxel_buffer: list[GlVoxel] = []
        self.buffer_mutex.unlock()

        self.window.run_widget.play.clicked.connect(self.run_function)
        self.window.tree.itemClicked.connect(self.tree_item_click)

    def run_function(self):
        run_options = get_run_options(self)

        run = run_options[self.window.run_widget.run_configs.currentText()][0]
        args = run_options[self.window.run_widget.run_configs.currentText()][1]
        self.thread_pool.start(run(*args))

    def update_view(self):
        view = pyrr.matrix44.create_look_at(pyrr.Vector3([self.grid.maxes[1] * self.zoom,
                                                          self.translate[0], self.translate[1]]),
                                            pyrr.Vector3([0, self.translate[0], self.translate[1]]),
                                            pyrr.Vector3([0, 1, 0]))
        glUniformMatrix4fv(self.view_loc, 1, GL_FALSE, view)

    def normalize_mouse_coords(self, a0):
        a_min = min([self.frameSize().width() / 2, self.frameSize().height() / 2])
        norm_x = (a0.x() - self.frameSize().width() / 2) / a_min
        norm_y = (a0.y() - self.frameSize().height() / 2) / -a_min

        return norm_x, norm_y

    def mouseMoveEvent(self, a0: QMouseEvent):
        norm_x, norm_y = self.normalize_mouse_coords(a0)
        self.vector = [norm_x - self.mouse[0], norm_y - self.mouse[1]]

        if self.is_rotate:
            self.theta[1] = self.theta_[1] - self.vector[0]
            self.theta[2] = self.theta_[2] + self.vector[1]

        elif self.is_translate:
            self.translate[0] = self.translate_[0] - self.vector[1] * 100
            self.translate[1] = self.translate_[1] - self.vector[0] * 100

            self.update_view()

        self.update()

    def mousePressEvent(self, a0: QMouseEvent):
        self.mouse = self.normalize_mouse_coords(a0)

        if a0.button() == Qt.MouseButton.RightButton:
            self.is_rotate = True
            self.theta_ = [rad for rad in self.theta]

        elif a0.button() == Qt.MouseButton.MiddleButton:
            self.is_translate = True
            self.translate_ = [vec for vec in self.translate]

    def mouseReleaseEvent(self, a0: QMouseEvent):
        if self.is_rotate:
            self.is_rotate = False
            self.theta_ = [rad for rad in self.theta]

        elif self.is_translate:
            self.is_translate = False
            self.translate_ = [vec for vec in self.translate]

        self.mouse = [0, 0]
        self.vector = [0, 0]

    def wheelEvent(self, a0: QWheelEvent):
        self.zoom = max(-32., min(-1e-4, self.zoom + -a0.angleDelta().y() / 120))
        self.update_view()
        self.update()

    def dragEnterEvent(self, e):
        if e.mimeData().hasText():
            files = [file.toLocalFile() for file in e.mimeData().urls() if
                     file.toLocalFile().split(".")[1].upper() in ["STL", "PNG", "JPG"]]
            if files:
                e.accept()
                return
        e.ignore()

    def dropEvent(self, e: QDropEvent):
        self.thread_pool.start(ImportRunnable(self, e.mimeData().urls()))

    def resizeGL(self, width: int, height: int):
        if height <= 0 or width <= 0:
            return

        glViewport(0, 0, width, height)
        projection_ = pyrr.matrix44.create_perspective_projection_matrix(45, width / height, 0.1, 10000)
        glUniformMatrix4fv(self.proj_loc, 1, GL_FALSE, projection_)

    def paintGL(self):
        if self.model_buffer or self.image_buffer or self.voxel_buffer:
            self.buffer_mutex.lock()

            if self.model_buffer:
                for model in self.model_buffer:
                    model.gl_calls()
                    self.models.append(model)
                self.model_buffer = []
            if self.image_buffer:
                for image in self.image_buffer:
                    image.gl_calls()
                    self.images.append(image)
                self.image_buffer = []
            if self.voxel_buffer:
                for voxel in self.voxel_buffer:
                    voxel.gl_calls()
                    self.voxels.append(voxel)
                self.voxel_buffer = []

            self.buffer_mutex.unlock()

            self.update_item_tree()
            self.grid = GlGrid(self)
            self.grid.gl_calls()

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        self.grid.draw()
        for model in self.models:
            model.draw()
        for image in self.images:
            image.draw()
        for voxel in self.voxels:
            voxel.draw()

        glFlush()

    def initializeGL(self):
        [model.gl_calls() for model in self.models]
        self.grid.gl_calls()
        [image.gl_calls() for image in self.images]

        self.model_loc, self.proj_loc, self.view_loc, self.shader = display_setup(*self.grid.maxes)

        self.update_item_tree()

    def update_item_tree(self):
        items = [QTreeWidgetItem([category]) for category in ["Environment", "Models", "Textures", "Voxels"]]

        grid_item = QTreeWidgetItem(["Grid"])
        grid_item.setIcon(0, QIcon(":grid.svg"))
        items[0].addChild(grid_item)
        for model in self.models:
            child = QTreeWidgetItem([model.filename])
            child.setIcon(0, QIcon(":model.svg"))
            items[1].addChild(child)
        for image in self.images:
            child = QTreeWidgetItem([image.filename])
            child.setIcon(0, QIcon(":image.svg"))
            items[2].addChild(child)
        for i, voxel in enumerate(self.voxels):
            child = QTreeWidgetItem(["voxels #%i" % i])
            child.setIcon(0, QIcon(":voxel.svg"))
            items[3].addChild(child)

        self.window.tree.clear()
        self.window.tree.insertTopLevelItems(0, items)
        self.window.tree.expandAll()
        self.window.tree.update()

    def tree_item_click(self, item: QTreeWidgetItem, column):
        model_dict = {model.filename: model for model in self.models}
        image_dict = {image.filename: image for image in self.images}

        labels = {"X": 0, "Y": 1, "Z": 2}

        if item.text(column) in model_dict:
            element = model_dict[item.text(column)]
        elif item.text(column) in image_dict:
            element = image_dict[item.text(column)]
        else:
            return

        def rotation_callback(value_label: str, value: float):
            element.theta[labels[value_label]] = value
            self.update()

        def position_callback(value_label: str, value: float):
            element.position[labels[value_label]] = value
            self.update()

        def scale_callback(value_label: str, value: float):
            element.scale[labels[value_label]] = value
            self.update()

        for property_callback, property_element, property_values in zip([rotation_callback,
                                                                         position_callback,
                                                                         scale_callback],
                                                                        [self.window.obj_rotation_edit,
                                                                         self.window.obj_location_edit,
                                                                         self.window.obj_scale_edit],
                                                                        [element.theta,
                                                                         element.position,
                                                                         element.scale]):
            for (_, widget), property_value in zip(property_element.properties_widgets.items(), property_values):
                try:
                    widget.value_edited.disconnect()
                except TypeError:
                    pass
                widget.value = property_value
                widget.update_text_box()
                widget.value_edited.connect(property_callback)

        self.window.property_label.setText(item.text(column))


if __name__ == '__main__':
    app = QApplication(sys.argv)

    w = MainWindow()
    app.exec_()



from pathlib import Path

import numpy as np
from PIL import Image

directory = "C:\\Users\\xia_t\\Desktop\\Main Folder\\minecraft\\assets\\minecraft\\"
blocks = [i.replace("\n", "") for i in open("one_dot_sixteen/one_dot_sixteen.txt", "r").readlines()]


def main():
    for block in blocks:
        mc_id, filename = block.split(",")

        paths = [path for path in Path(directory).rglob("%s*.png" % filename)]

        if len(paths) == 0:
            print("%s not found" % mc_id)
            continue

        image_average(paths[0].absolute(), mc_id)


def image_average(path, block):
    image = np.array(Image.open(path))

    if image.shape[-1] == 3:
        image = np.reshape(image, (-1, 3))
        alpha = np.repeat([255], image.shape[0])
        image = np.hstack((image, np.expand_dims(alpha, axis=1)))

    image = np.reshape(image, (-1, 4))

    alpha = image[:, 3]
    image = np.delete(image, 3, axis=1)

    average = np.round(np.average(image, axis=0, weights=alpha)).astype(int)

    print("\"minecraft:%s\": [%i, %i, %i]," % (block, average[0], average[1], average[2]))

    # if image.shape[-1] == 4:
    #     image = np.delete(image, 3, axis=2)
    #
    # image = np.reshape(image, (-1, 3))
    #
    # mean = np.mean(image, axis=0, dtype=int)
    # print("[\"minecraft:%s\",[%i,%i,%i]]" % (block, mean[0], mean[1], mean[2]))


if __name__ == '__main__':
    main()



import numpy as np
import pyrr
# noinspection PyPackageRequirements
from OpenGL.GL import *
# noinspection PyPackageRequirements
from OpenGL.GL.shaders import compileProgram, compileShader

from craftslicer.core.gl.gl_shaders import vertex_shader, fragment_shader
from craftslicer.core.models import cube


def vector_to_vertex_index(vectors, dims=3):
    vertices = [vertex for triangle in vectors for vertex in triangle]

    i = vectors.shape[0] * dims
    faces = list(np.linspace(0, i, i, endpoint=False).reshape((vectors.shape[0], dims)).astype(int))

    return vertices, faces


# noinspection DuplicatedCode
def vector_gl(vectors):
    face_vertices, indices = vector_to_vertex_index(vectors)

    indices = np.array(indices, dtype=int)
    face_vertices = np.array(face_vertices, dtype=np.float32)[indices]

    face_vertices = np.reshape(face_vertices, (-1, 3))

    x_min, y_min, z_min = np.amin(face_vertices, axis=0)
    face_vertices[:, 0] -= x_min
    face_vertices[:, 1] -= y_min
    face_vertices[:, 2] -= z_min
    vertices = face_vertices

    face_colors = np.repeat([[0.7, 0.7, 0.7, 1.0]], face_vertices.shape[0], axis=0)
    line_colors = np.repeat([[1.0, 1.0, 1.0, 1.0]], face_vertices.shape[0], axis=0)

    line_vertices = np.concatenate((face_vertices, line_colors), axis=1)
    line_vertices = np.reshape(line_vertices, (-1, 3, 7))
    q, _ = divmod(line_vertices.shape[0], 10000)
    line_vertices = line_vertices[0::q + 1]
    line_vertices = line_vertices.reshape((-1, 7))
    face_vertices = np.concatenate((face_vertices, face_colors), axis=1)

    x_max, y_max, z_max = np.amax(vertices, axis=0)
    for vertices_ in [face_vertices, line_vertices]:
        for column, max_ in zip([0, 1, 2], [x_max, y_max, z_max]):
            vertices_[:, column] -= max_ / 2

    face_vertices, indices = face_vertices.flatten(), indices.flatten()

    return face_vertices.astype(np.float32), line_vertices.flatten().astype(np.float32), vertices, indices.astype(
        np.uint32)


def image_gl(image, maxes):
    width, height = image.size
    aspect = width / height
    width, height = int(maxes[1]), int(maxes[1] / aspect)

    image = image.resize((width, height))
    image = np.flip(image, axis=0)

    vectors = np.array([
        [[[0, y_, x_], [0, y_, x_ + 1], [0, y_ + 1, x_]],
         [[0, y_, x_ + 1], [0, y_ + 1, x_], [0, y_ + 1, x_ + 1]]]
        for y_ in range(image.shape[1]) for x_ in range(image.shape[0])
    ]).reshape((-1, 3, 3))

    image_colors = image.copy().astype(np.float32) / 255
    colors = np.array([
        [image_colors[x_][y_] for _ in range(6)]
        for y_ in range(image.shape[1]) for x_ in range(image.shape[0])
    ]).reshape((-1, image.shape[-1]))

    if image.shape[-1] == 3:
        alpha = np.ones(image.flatten().shape[0] * 2, dtype=np.float32) * 0.5
        colors = np.insert(colors, image.shape[-1], alpha, axis=1)
    colors[:, 3] = 0.5

    vertices, indices = vector_to_vertex_index(vectors)
    vertices, indices = np.array(vertices), np.array(indices)
    vertices = np.concatenate((vertices, colors), axis=1)

    for column, max_ in zip([0, 1, 2], maxes):
        vertices[:, column] -= max_ / 2

    return vertices.flatten().astype(np.float32), indices.flatten().astype(np.uint32), (width, height), image


def voxel_gl(voxels, maxes, voxel_color: np.ndarray = None):
    cube_vertices, cube_indices = cube
    cube_vertices, cube_indices = np.array(cube_vertices).astype(float), np.array(cube_indices).astype(int)

    vectors = cube_vertices[cube_indices].reshape((-1, 3))
    voxel_indices = np.argwhere(voxels == 1)
    vectors = np.tile(vectors, (voxel_indices.shape[0], 1))
    voxel_indices = np.repeat(voxel_indices, 36, axis=0)
    vectors = vectors + voxel_indices

    vertices, indices = vector_to_vertex_index(vectors)
    vertices, indices = np.array(vertices).reshape((-1, 3)), np.array(indices)

    if voxel_color is not None:
        face_colors = voxel_color.copy()
        voxel_indices = np.argwhere(voxels.flatten() == 1)
        face_colors = face_colors.reshape((-1, 3))
        face_colors = np.pad(face_colors, ((0, 0), (0, 1)), constant_values=255)
        face_colors = face_colors.astype(np.float32)
        face_colors = face_colors[voxel_indices.flatten()]
        face_colors /= 255
        face_colors = np.repeat(face_colors, 36, axis=0)
        line_colors = face_colors
    else:
        face_colors = np.repeat([[0.7, 0.7, 0.7, 1.0]], vertices.shape[0], axis=0)
        line_colors = np.repeat([[1.0, 1.0, 1.0, 1.0]], vertices.shape[0], axis=0)

    face_vertices = np.concatenate((vertices, face_colors), axis=1)
    line_vertices = np.concatenate((vertices, line_colors), axis=1)

    for vertices_ in [face_vertices, line_vertices]:
        for column, max_ in zip([0, 1, 2], maxes):
            vertices_[:, column] -= max_ / 2

    return face_vertices.astype(np.float32), line_vertices.flatten().astype(np.float32), vertices, indices.astype(
        np.uint32)


# noinspection DuplicatedCode
def grid_gl(x_max, y_max, z_max):
    xy_max = max([x_max, y_max])
    xy_max = int(xy_max / np.sin(np.pi / 4))
    q, r = divmod(xy_max, 10)
    xy_max = (q + 1) * 10

    grid_x = np.array([[[i, 0, 0], [i, xy_max, 0]] for i in range(0, xy_max + 10, 10)])
    grid_y = np.array([[[0, i, 0], [xy_max, i, 0]] for i in range(0, xy_max + 10, 10)])
    grid_z = np.array([[[0, 0, 0], [0, 0, z_max]]])
    grid_vertices, grid_indices = vector_to_vertex_index(np.concatenate((grid_x, grid_y, grid_z)), dims=2)
    grid_indices = np.array(grid_indices, dtype=int)
    grid_vertices = np.array(grid_vertices, dtype=np.float32)[grid_indices]
    grid_vertices = np.reshape(grid_vertices, (-1, 3))

    grid_color_values = np.repeat([[1.0, 1.0, 1.0, 1.0]], grid_vertices.shape[0], axis=0)
    grid_color_values[0] = [0.0, 1.0, 0.0, 1.0]
    grid_color_values[1] = [0.0, 1.0, 0.0, 1.0]
    grid_color_values[grid_x.reshape((-1, 3)).shape[0]] = [1.0, 0.0, 0.0, 1.0]
    grid_color_values[grid_x.reshape((-1, 3)).shape[0] + 1] = [1.0, 0.0, 0.0, 1.0]
    grid_color_values[grid_color_values.shape[0] - 1] = [0.0, 0.0, 1.0, 1.0]
    grid_color_values[grid_color_values.shape[0] - 2] = [0.0, 0.0, 1.0, 1.0]

    for column, max_ in zip([0, 1, 2], [xy_max, xy_max, z_max]):
        grid_vertices[:, column] -= max_ / 2

    grid_vertices = np.concatenate((grid_vertices, grid_color_values), axis=1)
    grid_vertices, grid_indices = grid_vertices.astype(np.float32), grid_indices.astype(np.uint32)

    return grid_vertices.flatten(), grid_indices.flatten(), [xy_max, xy_max, z_max]


# noinspection DuplicatedCode
def write_vao(vao_id, vbo_id, ebo_id, vertices, indices):
    glBindVertexArray(vao_id)

    glBindBuffer(GL_ARRAY_BUFFER, vbo_id)
    glBufferData(GL_ARRAY_BUFFER, vertices.nbytes, vertices, GL_STATIC_DRAW)

    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ebo_id)
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.nbytes, indices, GL_STATIC_DRAW)

    glEnableVertexAttribArray(0)
    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, vertices.itemsize * 7, ctypes.c_void_p(0))
    glEnableVertexAttribArray(1)
    glVertexAttribPointer(1, 4, GL_FLOAT, GL_FALSE, vertices.itemsize * 7, ctypes.c_void_p(12))

    glBindVertexArray(0)


def create_vao(*args):
    vao = glGenVertexArrays(len(args))
    vbo = glGenBuffers(len(args))
    ebo = glGenBuffers(len(args))

    if len(args) == 1:
        write_vao(vao, vbo, ebo, *args[0])
    else:
        for i, (vertices, indices) in enumerate(args):
            write_vao(vao[i], vbo[i], ebo[i], vertices, indices)

    return vao, vbo, ebo


def compile_shader(vertex_src, fragment_src):
    return compileProgram(compileShader(vertex_src, GL_VERTEX_SHADER),
                          compileShader(fragment_src, GL_FRAGMENT_SHADER))


# noinspection DuplicatedCode
def display_setup(x_max, y_max, _):
    shader = compile_shader(vertex_shader, fragment_shader)
    glUseProgram(shader)

    glClearColor(80 / 255, 91 / 255, 103 / 255, 1)

    glEnable(GL_DEPTH_TEST)
    glEnable(GL_BLEND)
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

    projection = pyrr.matrix44.create_perspective_projection_matrix(45, 1280 / 720, 0.1, x_max * 2)

    view = pyrr.matrix44.create_look_at(pyrr.Vector3([y_max * -6, 0, 0]),
                                        pyrr.Vector3([0, 0, 0]),
                                        pyrr.Vector3([0, 1, 0]))

    model_loc = glGetUniformLocation(shader, "model")
    proj_loc = glGetUniformLocation(shader, "projection")
    view_loc = glGetUniformLocation(shader, "view")

    glUniformMatrix4fv(proj_loc, 1, GL_FALSE, projection)
    glUniformMatrix4fv(view_loc, 1, GL_FALSE, view)

    return model_loc, proj_loc, view_loc, shader


def position_matrix(theta, pos, scale, reverse=False):
    pos_matrix = pyrr.matrix44.create_from_translation(pyrr.Vector3(pos))
    scale_matrix = pyrr.matrix44.create_from_scale(pyrr.Vector3(scale))
    rot_x = pyrr.Matrix44.from_x_rotation(theta[0])
    rot_y = pyrr.Matrix44.from_y_rotation(theta[1])
    rot_z = pyrr.Matrix44.from_z_rotation(theta[2])

    if reverse:
        return scale_matrix @ pos_matrix @ rot_z @ rot_y @ rot_x
    return rot_x @ rot_y @ rot_z @ pos_matrix @ scale_matrix



# noinspection SpellCheckingInspection
palette = \
    {
        "minecraft:air": [0, 0, 0],
        # "minecraft:acacia_log": [97, 97, 87],
        # "minecraft:acacia_planks": [90, 90, 50],
        "minecraft:ancient_debris": [96, 64, 56],
        "minecraft:andesite": [136, 136, 137],
        "minecraft:barrel": [122, 90, 52],
        "minecraft:basalt": [73, 73, 78],
        "minecraft:bedrock": [85, 85, 85],
        "minecraft:bee_nest": [161, 127, 88],
        "minecraft:birch_log": [217, 215, 210],
        "minecraft:birch_planks": [192, 175, 121],
        "minecraft:black_concrete": [8, 10, 15],
        "minecraft:black_concrete_powder": [25, 27, 32],
        "minecraft:black_glazed_terracotta": [68, 30, 32],
        "minecraft:black_terracotta": [37, 23, 16],
        "minecraft:black_stained_glass": [25, 25, 25],
        "minecraft:black_wool": [21, 21, 26],
        "minecraft:blackstone": [42, 35, 41],
        "minecraft:blast_furnace": [108, 108, 107],
        "minecraft:blue_concrete": [45, 47, 143],
        "minecraft:blue_concrete_powder": [70, 73, 167],
        "minecraft:blue_glazed_terracotta": [47, 65, 139],
        "minecraft:blue_ice": [116, 168, 253],
        "minecraft:blue_terracotta": [74, 60, 91],
        "minecraft:blue_stained_glass": [51, 76, 178],
        "minecraft:blue_wool": [53, 57, 157],
        "minecraft:bone_block": [229, 226, 208],
        "minecraft:bone_meal": [159, 158, 180],
        "minecraft:bookshelf": [117, 95, 60],
        "minecraft:brain_coral": [198, 85, 152],
        "minecraft:brain_coral_block": [207, 91, 159],
        "minecraft:bricks": [151, 98, 83],
        "minecraft:brown_concrete": [96, 60, 32],
        "minecraft:brown_concrete_powder": [126, 85, 54],
        "minecraft:brown_glazed_terracotta": [120, 106, 86],
        "minecraft:brown_stained_glass": [102, 76, 51],
        "minecraft:brown_mushroom_block": [149, 112, 81],
        "minecraft:brown_terracotta": [77, 51, 36],
        "minecraft:brown_wool": [114, 72, 41],
        "minecraft:bubble_coral": [161, 24, 160],
        "minecraft:bubble_coral_block": [165, 26, 162],
        "minecraft:cartography_table": [72, 50, 34],
        "minecraft:carved_pumpkin": [150, 84, 17],
        "minecraft:chiseled_nether_bricks": [47, 24, 28],
        "minecraft:chiseled_polished_blackstone": [54, 49, 57],
        "minecraft:chiseled_quartz_block": [232, 227, 218],
        "minecraft:chiseled_red_sandstone": [183, 97, 28],
        "minecraft:chiseled_sandstone": [216, 203, 155],
        "minecraft:chiseled_stone_bricks": [120, 119, 120],
        "minecraft:coal_block": [16, 16, 16],
        "minecraft:coal_ore": [116, 116, 116],
        "minecraft:coarse_dirt": [119, 86, 59],
        "minecraft:cobblestone": [128, 127, 128],
        "minecraft:cracked_nether_bricks": [40, 20, 24],
        "minecraft:cracked_polished_blackstone_bricks": [44, 38, 44],
        "minecraft:cracked_stone_bricks": [118, 118, 118],
        "minecraft:crimson_nylium": [131, 31, 31],
        "minecraft:crimson_planks": [101, 49, 71],
        "minecraft:crimson_stem": [93, 26, 30],
        "minecraft:crying_obsidian": [33, 10, 60],
        "minecraft:cut_red_sandstone": [189, 102, 32],
        "minecraft:cut_sandstone": [218, 206, 160],
        "minecraft:cyan_concrete": [21, 119, 136],
        "minecraft:cyan_concrete_powder": [37, 148, 157],
        "minecraft:cyan_glazed_terracotta": [52, 119, 125],
        "minecraft:cyan_stained_glass": [76, 127, 153],
        "minecraft:cyan_terracotta": [87, 91, 91],
        "minecraft:cyan_wool": [21, 138, 145],
        "minecraft:dark_oak_log": [60, 47, 26],
        "minecraft:dark_oak_planks": [67, 43, 20],
        "minecraft:dark_prismarine": [52, 92, 76],
        "minecraft:dead_brain_coral_block": [124, 118, 114],
        "minecraft:dead_bubble_coral_block": [132, 124, 119],
        "minecraft:dead_fire_coral_block": [132, 124, 120],
        "minecraft:dead_horn_coral_block": [134, 126, 122],
        "minecraft:dead_tube_coral_block": [130, 123, 120],
        "minecraft:diorite": [189, 188, 189],
        "minecraft:dirt": [134, 96, 67],
        "minecraft:dried_kelp_block": [38, 49, 30],
        "minecraft:emerald_block": [42, 203, 88],
        "minecraft:emerald_ore": [117, 137, 124],
        "minecraft:end_stone": [220, 223, 158],
        "minecraft:end_stone_bricks": [218, 224, 162],
        "minecraft:fire_coral_block": [164, 35, 47],
        "minecraft:fletching_table": [173, 155, 111],
        "minecraft:gilded_blackstone": [57, 44, 39],
        "minecraft:glass": [176, 214, 219],
        "minecraft:glowstone": [172, 131, 84],
        "minecraft:gold_block": [246, 208, 62],
        "minecraft:gold_ore": [144, 140, 125],
        "minecraft:granite": [149, 103, 86],
        "minecraft:grass_block": [127, 107, 66],
        "minecraft:grass_path": [142, 106, 70],
        "minecraft:gravel": [132, 127, 127],
        "minecraft:gray_concrete": [55, 58, 62],
        "minecraft:gray_concrete_powder": [77, 81, 85],
        "minecraft:gray_glazed_terracotta": [83, 90, 94],
        "minecraft:gray_stained_glass": [76, 76, 76],
        "minecraft:gray_terracotta": [58, 42, 36],
        "minecraft:gray_wool": [63, 68, 72],
        "minecraft:green_concrete": [73, 91, 36],
        "minecraft:green_concrete_powder": [97, 119, 45],
        "minecraft:green_glazed_terracotta": [117, 142, 67],
        "minecraft:green_stained_glass": [102, 127, 51],
        "minecraft:green_terracotta": [76, 83, 42],
        "minecraft:green_wool": [85, 110, 28],
        "minecraft:hay_block": [166, 136, 38],
        "minecraft:honey_block": [241, 146, 18],
        "minecraft:honeycomb_block": [229, 148, 30],
        "minecraft:horn_coral_block": [216, 200, 66],
        "minecraft:ice": [146, 184, 254],
        "minecraft:iron_block": [220, 220, 220],
        "minecraft:iron_ore": [136, 131, 127],
        "minecraft:jack_o_lantern": [215, 152, 53],
        "minecraft:jukebox": [4, 5, 5],
        "minecraft:jungle_log": [85, 68, 25],
        "minecraft:jungle_planks": [160, 115, 81],
        "minecraft:lapis_block": [31, 67, 140],
        "minecraft:lapis_ore": [99, 111, 133],
        "minecraft:light_blue_concrete": [36, 137, 199],
        "minecraft:light_blue_concrete_powder": [74, 181, 213],
        "minecraft:light_blue_glazed_terracotta": [95, 165, 209],
        "minecraft:light_blue_stained_glass": [102, 153, 216],
        "minecraft:light_blue_terracotta": [113, 109, 138],
        "minecraft:light_blue_wool": [58, 175, 217],
        "minecraft:light_gray_concrete": [125, 125, 115],
        "minecraft:light_gray_concrete_powder": [155, 155, 148],
        "minecraft:light_gray_glazed_terracotta": [144, 166, 168],
        "minecraft:light_gray_stained_glass": [153, 153, 153],
        "minecraft:light_gray_terracotta": [135, 107, 98],
        "minecraft:light_gray_wool": [142, 142, 135],
        "minecraft:lime_concrete": [94, 169, 24],
        "minecraft:lime_concrete_powder": [125, 189, 42],
        "minecraft:lime_glazed_terracotta": [163, 198, 55],
        "minecraft:lime_stained_glass": [127, 204, 25],
        "minecraft:lime_terracotta": [104, 118, 53],
        "minecraft:lime_wool": [112, 185, 26],
        "minecraft:lodestone": [119, 120, 123],
        "minecraft:loom": [76, 60, 36],
        "minecraft:magenta_concrete": [169, 48, 159],
        "minecraft:magenta_concrete_powder": [193, 84, 185],
        "minecraft:magenta_glazed_terracotta": [208, 100, 192],
        "minecraft:magenta_stained_glass": [178, 76, 216],
        "minecraft:magenta_terracotta": [150, 88, 109],
        "minecraft:magenta_wool": [190, 69, 180],
        "minecraft:magma_block": [143, 63, 32],
        "minecraft:mossy_cobblestone": [110, 118, 95],
        "minecraft:mossy_stone_bricks": [115, 121, 105],
        "minecraft:mushroom_stem": [203, 197, 186],
        "minecraft:mycelium": [113, 87, 71],
        "minecraft:nether_brick": [3, 5, 4],
        "minecraft:nether_bricks": [3, 5, 4],
        "minecraft:nether_gold_ore": [115, 55, 42],
        "minecraft:nether_quartz_ore": [118, 66, 62],
        "minecraft:nether_wart_block": [115, 3, 2],
        "minecraft:netherite_block": [67, 61, 64],
        "minecraft:netherrack": [98, 38, 38],
        "minecraft:oak_log": [109, 85, 51],
        "minecraft:oak_planks": [162, 131, 79],
        "minecraft:obsidian": [15, 11, 25],
        "minecraft:orange_concrete": [224, 97, 1],
        "minecraft:orange_concrete_powder": [227, 132, 32],
        "minecraft:orange_glazed_terracotta": [155, 147, 92],
        "minecraft:orange_stained_glass": [216, 127, 51],
        "minecraft:orange_terracotta": [162, 84, 38],
        "minecraft:orange_wool": [241, 118, 20],
        "minecraft:packed_ice": [142, 180, 250],
        "minecraft:pink_concrete": [214, 101, 143],
        "minecraft:pink_concrete_powder": [229, 153, 181],
        "minecraft:pink_glazed_terracotta": [235, 155, 182],
        "minecraft:pink_stained_glass": [242, 127, 165],
        "minecraft:pink_terracotta": [162, 78, 79],
        "minecraft:pink_wool": [238, 141, 172],
        "minecraft:podzol": [123, 88, 57],
        "minecraft:polished_andesite": [132, 135, 134],
        "minecraft:polished_basalt": [89, 88, 92],
        "minecraft:polished_blackstone": [53, 49, 57],
        "minecraft:polished_blackstone_bricks": [47, 41, 48],
        "minecraft:polished_diorite": [193, 193, 195],
        "minecraft:polished_granite": [154, 107, 89],
        "minecraft:prismarine": [99, 156, 151],
        "minecraft:prismarine_bricks": [99, 172, 158],
        "minecraft:pumpkin": [196, 115, 24],
        "minecraft:purple_concrete": [100, 32, 156],
        "minecraft:purple_concrete_powder": [132, 56, 178],
        "minecraft:purple_glazed_terracotta": [110, 48, 152],
        "minecraft:purple_stained_glass": [127, 63, 178],
        "minecraft:purple_terracotta": [118, 70, 86],
        "minecraft:purple_wool": [122, 42, 173],
        "minecraft:purpur_block": [170, 126, 170],
        "minecraft:purpur_pillar": [172, 130, 172],
        "minecraft:quartz_block": [237, 230, 224],
        "minecraft:quartz_bricks": [235, 229, 222],
        "minecraft:quartz_pillar": [236, 231, 224],
        "minecraft:red_concrete": [142, 33, 33],
        "minecraft:red_concrete_powder": [168, 54, 51],
        "minecraft:red_glazed_terracotta": [182, 60, 53],
        "minecraft:red_stained_glass": [153, 51, 51],
        "minecraft:red_mushroom": [217, 75, 68],
        "minecraft:red_mushroom_block": [200, 47, 45],
        "minecraft:red_nether_bricks": [70, 7, 9],
        "minecraft:red_sand": [191, 103, 33],
        "minecraft:red_sandstone": [187, 99, 29],
        "minecraft:red_terracotta": [143, 61, 47],
        "minecraft:red_wool": [161, 39, 35],
        "minecraft:redstone_block": [4, 3, 4],
        "minecraft:redstone_lamp": [95, 55, 30],
        "minecraft:redstone_ore": [133, 108, 108],
        "minecraft:sand": [219, 207, 163],
        "minecraft:sandstone": [216, 203, 156],
        "minecraft:sea_lantern": [172, 200, 190],
        "minecraft:shroomlight": [241, 147, 71],
        "minecraft:slime_block": [2, 2, 2],
        "minecraft:smithing_table": [64, 28, 24],
        "minecraft:smoker": [107, 106, 104],
        "minecraft:smooth_stone": [159, 159, 159],
        "minecraft:snow_block": [249, 254, 254],
        "minecraft:soul_sand": [81, 62, 51],
        "minecraft:soul_soil": [76, 58, 47],
        "minecraft:spruce_log": [59, 38, 17],
        "minecraft:spruce_planks": [115, 85, 49],
        "minecraft:stone": [126, 126, 126],
        "minecraft:stone_bricks": [122, 122, 122],
        "minecraft:stripped_acacia_log": [175, 93, 60],
        "minecraft:stripped_birch_log": [197, 176, 118],
        "minecraft:stripped_crimson_stem": [137, 57, 90],
        "minecraft:stripped_dark_oak_log": [97, 76, 50],
        "minecraft:stripped_jungle_log": [171, 133, 85],
        "minecraft:stripped_oak_log": [177, 144, 86],
        "minecraft:stripped_spruce_log": [116, 90, 52],
        "minecraft:stripped_warped_stem": [58, 151, 148],
        "minecraft:target": [229, 176, 168],
        "minecraft:terracotta": [152, 94, 68],
        "minecraft:tube_coral_block": [49, 87, 207],
        "minecraft:warped_fungus": [74, 109, 88],
        "minecraft:warped_nylium": [43, 114, 101],
        "minecraft:warped_planks": [43, 105, 99],
        "minecraft:warped_wart_block": [23, 120, 121],
        "minecraft:wet_sponge": [171, 181, 70],
        "minecraft:white_concrete": [207, 213, 214],
        "minecraft:white_concrete_powder": [226, 227, 228],
        "minecraft:white_glazed_terracotta": [188, 212, 203],
        "minecraft:white_stained_glass": [255, 255, 255],
        "minecraft:white_terracotta": [210, 178, 161],
        "minecraft:white_wool": [234, 236, 237],
        "minecraft:yellow_concrete": [241, 175, 21],
        "minecraft:yellow_concrete_powder": [233, 199, 55],
        "minecraft:yellow_glazed_terracotta": [234, 192, 89],
        "minecraft:yellow_stained_glass": [229, 229, 51],
        "minecraft:yellow_terracotta": [186, 133, 35],
        "minecraft:yellow_wool": [249, 198, 40]
    }



from PyQt5.QtCore import pyqtSignal
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *

# noinspection PyUnresolvedReferences
from craftslicer.core.res import qrc_resources
from craftslicer.core.qt.qt_threads import get_run_options


class ExpandConstraint(QWidget):
    def __init__(self):
        super(ExpandConstraint, self).__init__()
        self.align = QWidget()
        self.align.setSizePolicy(QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding))
        self.align.setContentsMargins(0, 0, 0, 0)

        self.parent_layout = QHBoxLayout(self)
        self.parent_layout.addWidget(self.align)


class SeparatorLine(QWidget):
    def __init__(self):
        super(SeparatorLine, self).__init__()
        self.line = QFrame()
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)
        self.line.setContentsMargins(0, 0, 0, 0)

        self.parent_layout = QHBoxLayout(self)
        self.parent_layout.addWidget(self.line)


# noinspection PyUnresolvedReferences
class SpinBox(QWidget):
    value_edited = pyqtSignal(str, float)

    def __init__(self, label="", default_value=0, button_step=0.1):
        super(SpinBox, self).__init__()
        self.value = default_value
        self.value_label = label
        self.button_step = button_step

        self.parent_layout = QHBoxLayout(self)

        self.tag = QLabel()
        self.tag.setText(label)
        self.parent_layout.addWidget(self.tag)

        self.decrement = QPushButton()
        self.decrement.clicked.connect(self.decrement_signal)
        self.decrement.setIcon(QIcon(":arrow_left.svg"))
        self.parent_layout.addWidget(self.decrement)

        self.text_box = QLineEdit()
        self.update_text_box()
        self.text_box.textEdited.connect(self.text_edited)
        self.text_box.setValidator(QDoubleValidator(-1024., 1024., 3))
        self.parent_layout.addWidget(self.text_box)

        self.increment = QPushButton()
        self.increment.clicked.connect(self.increment_signal)
        self.increment.setIcon(QIcon(":arrow_right.svg"))
        self.parent_layout.addWidget(self.increment)

        self.lock = QPushButton()
        self.lock.setIcon(QIcon(":lock_closed.svg"))
        self.parent_layout.addWidget(self.lock)

        self.parent_layout.setContentsMargins(0, 0, 0, 0)

    def update_text_box(self):
        self.text_box.setText("%0.3f" % self.value)

    def text_edited(self, text: str):
        try:
            self.value = float(text)
        except ValueError:
            self.value = 0.

        self.value_edited.emit(self.value_label, self.value)

    def increment_signal(self):
        self.value += self.button_step
        self.update_text_box()
        self.value_edited.emit(self.value_label, self.value)

    def decrement_signal(self):
        self.value -= self.button_step
        self.update_text_box()
        self.value_edited.emit(self.value_label, self.value)


class PropertiesEdit(QWidget):
    def __init__(self, property_label="", default_value=0, button_step=0.1):
        super(PropertiesEdit, self).__init__()
        self.properties_widgets = {label: SpinBox(label, default_value, button_step) for label in ["X", "Y", "Z"]}

        self.property_label_layout = QVBoxLayout()
        self.property_label = QLabel(property_label)
        self.property_label_layout.addWidget(self.property_label)

        self.property_widgets_layout = QVBoxLayout()
        [self.property_widgets_layout.addWidget(widget) for _, widget in self.properties_widgets.items()]

        self.parent_layout = QGridLayout(self)
        self.parent_layout.addLayout(self.property_label_layout, 0, 0)
        self.parent_layout.addLayout(self.property_widgets_layout, 0, 1)
        self.parent_layout.setColumnStretch(0, 1)
        self.parent_layout.setColumnStretch(1, 2)

        self.parent_layout.setContentsMargins(0, 0, 0, 0)


class RunWidget(QWidget):
    def __init__(self):
        super(RunWidget, self).__init__()
        self.parent_layout = QHBoxLayout(self)

        self.run_configs = QComboBox()
        # self.run_configs.addItems(["Run all", "Convert voxels", "Texture voxels"])
        # self.run_configs.addItems(["Convert voxels", "Texture voxels", "Assign Blocks", "Deploy"])
        self.run_configs.addItems(list(get_run_options(None).keys()))
        self.parent_layout.addWidget(self.run_configs)

        self.play = QPushButton()
        self.play.setIcon(QIcon(":play.svg"))
        self.parent_layout.addWidget(self.play)

        self.parent_layout.setContentsMargins(0, 0, 0, 0)



from PyQt5.QtCore import Qt, QRunnable, QThread
from PyQt5.QtWidgets import *

from craftslicer.core.gl.gl_elements import GlModel, GlImage, GlVoxel
from craftslicer.core.mc.palette_list import palette_list
from craftslicer.core.model_processor import convert_voxels, texture_voxels, assign_blocks, deploy_blocks
from craftslicer.core.qt.qt_dialog import AssignBlocksDialog, DeployBlocksDialog


def get_run_options(gl_widget):
    return {
        "Run All": [AllRunnable, [gl_widget]],
        "Convert Voxels": [ConvertVoxelsRunnable, [gl_widget]],
        "Texture Voxels": [TextureVoxelsRunnable, [gl_widget]],
        "Assign Blocks": [AssignBlocksRunnable, [gl_widget]],
        "Deploy": [DeployBlocksRunnable, [gl_widget]]
    }


class Runnable(QRunnable):
    def __init__(self, thread, thread_args, progress_dialog_label, progress_dialog_max):
        super().__init__()

        self.thread = thread(*thread_args)

        self.progress_dialog = QProgressDialog(progress_dialog_label, "Cancel", 0, progress_dialog_max)
        self.progress_dialog.setValue(0)

        # noinspection PyTypeChecker
        self.progress_dialog.setWindowFlags(
            self.progress_dialog.windowFlags() &
            Qt.CustomizeWindowHint | Qt.WindowTitleHint)

        self.progress_dialog.canceled.connect(self.cancel)
        self.progress_dialog.show()

    def run(self):
        self.thread.start()
        self.thread.wait()

    def cancel(self):
        self.thread.terminate()
        self.thread.wait()

        self.progress_dialog.reset()
        self.progress_dialog.close()


class ImportRunnable(Runnable):
    def __init__(self, gl_widget, files):
        files = [file.toLocalFile() for file in files if
                 file.toLocalFile().split(".")[1].upper() in ["STL", "PNG", "JPG"]]
        if not files:
            return

        class Thread(QThread):
            def __init__(self, gl_widget_, files_):
                super(Thread, self).__init__()

                self.gl_widget = gl_widget_
                self.files = files_

            def run(self):
                self.gl_widget.buffer_mutex.lock()

                for file in self.files:
                    if file.split(".")[1].upper() in ["STL"]:
                        self.gl_widget.model_buffer.append(GlModel(file, self.gl_widget))
                    elif file.split(".")[1].upper() in ["PNG", "JPG"]:
                        self.gl_widget.image_buffer.append(GlImage(file, self.gl_widget))

                self.gl_widget.buffer_mutex.unlock()

        super(ImportRunnable, self).__init__(Thread, [gl_widget, files],
                                             "".join("{0}\n".format(file) for file in files), 0)


class AllRunnable(Runnable):
    def __init__(self, gl_widget):
        dialog = AssignBlocksDialog()
        ret = dialog.exec_()
        if ret == QMessageBox.Cancel or ret == QMessageBox.Close:
            return

        palette = palette_list[dialog.version_list.currentText()]
        if not palette:
            return

        dialog = DeployBlocksDialog()
        ret = dialog.exec_()
        if ret == QMessageBox.Cancel or ret == QMessageBox.Close:
            return

        user_info = {
            "Author": dialog.author_text.text(),
            "Name": dialog.project_text.text()
        }

        class Thread(QThread):
            def __init__(self, gl_widget_, palette_, user_info_):
                super(Thread, self).__init__()

                self.running = True
                self.gl_widget = gl_widget_
                self.palette = palette_
                self.user_info = user_info_

            def run(self):
                voxels = convert_voxels(self.gl_widget.models, self.gl_widget.grid)
                gl_voxel = GlVoxel(voxels, self.gl_widget)

                voxel_color = texture_voxels(gl_voxel, self.gl_widget.images)
                gl_voxel = GlVoxel(voxels, self.gl_widget, voxel_color=voxel_color)

                gl_voxel.flattened_blocks = assign_blocks(voxels=voxels,
                                                          voxel_color=voxel_color,
                                                          palette=self.palette)

                deploy_blocks(voxels=voxels,
                              flattened_blocks=gl_voxel.flattened_blocks,
                              user_info=self.user_info)

                self.gl_widget.buffer_mutex.lock()
                self.gl_widget.voxels = []
                self.gl_widget.voxel_buffer.append(gl_voxel)
                self.gl_widget.buffer_mutex.unlock()

        super(AllRunnable, self).__init__(Thread, [gl_widget, palette, user_info], "Run All", 0)


class ConvertVoxelsRunnable(Runnable):
    def __init__(self, gl_widget):
        class Thread(QThread):
            def __init__(self, gl_widget_):
                super(Thread, self).__init__()

                self.gl_widget = gl_widget_

            def run(self):
                voxel = convert_voxels(self.gl_widget.models, self.gl_widget.grid)

                self.gl_widget.buffer_mutex.lock()
                self.gl_widget.voxels = []
                self.gl_widget.voxel_buffer.append(GlVoxel(voxel, self.gl_widget))
                self.gl_widget.buffer_mutex.unlock()

        super(ConvertVoxelsRunnable, self).__init__(Thread, [gl_widget], "Converting voxels", 0)


class TextureVoxelsRunnable(Runnable):
    def __init__(self, gl_widget):
        class Thread(QThread):
            def __init__(self, gl_widget_):
                super(Thread, self).__init__()

                self.gl_widget = gl_widget_

            def run(self):
                if not self.gl_widget.voxels:
                    return

                voxel_color = texture_voxels(self.gl_widget.voxels[0], self.gl_widget.images)
                voxels = self.gl_widget.voxels[0].voxels

                self.gl_widget.buffer_mutex.lock()
                self.gl_widget.voxels = []
                self.gl_widget.voxel_buffer.append(GlVoxel(voxels, self.gl_widget, voxel_color=voxel_color))
                self.gl_widget.buffer_mutex.unlock()

        super(TextureVoxelsRunnable, self).__init__(Thread, [gl_widget], "Texturing voxels", 0)


class AssignBlocksRunnable(Runnable):
    def __init__(self, gl_widget):
        dialog = AssignBlocksDialog()
        ret = dialog.exec_()

        if ret == QMessageBox.Cancel or ret == QMessageBox.Close:
            return

        palette = palette_list[dialog.version_list.currentText()]
        if not palette:
            return

        class Thread(QThread):
            def __init__(self, gl_widget_: gl_widget, palette_):
                super(Thread, self).__init__()

                self.gl_widget = gl_widget_
                self.palette = palette_

            def run(self):
                if not self.gl_widget.voxels:
                    return
                if self.gl_widget.voxels[0].voxel_color is None:
                    return

                self.gl_widget.voxels[0].flattened_blocks = assign_blocks(voxels=self.gl_widget.voxels[0].voxels,
                                                                          voxel_color=self.gl_widget.voxels[
                                                                              0].voxel_color,
                                                                          palette=self.palette)

        super(AssignBlocksRunnable, self).__init__(Thread, [gl_widget, palette], "Assigning Blocks", 0)


class DeployBlocksRunnable(Runnable):
    def __init__(self, gl_widget):
        dialog = DeployBlocksDialog()
        ret = dialog.exec_()

        if ret == QMessageBox.Cancel or ret == QMessageBox.Close:
            return

        user_info = {
            "Author": dialog.author_text.text(),
            "Name": dialog.project_text.text()
        }

        class Thread(QThread):
            def __init__(self, gl_widget_, user_info_):
                super(Thread, self).__init__()

                self.gl_widget = gl_widget_
                self.user_info = user_info_

            def run(self):
                if not self.gl_widget.voxels:
                    return
                if self.gl_widget.voxels[0].flattened_blocks is None:
                    return

                deploy_blocks(voxels=self.gl_widget.voxels[0].voxels,
                              flattened_blocks=self.gl_widget.voxels[0].flattened_blocks,
                              user_info=self.user_info)

        super(DeployBlocksRunnable, self).__init__(Thread, [gl_widget, user_info], "Deploying", 0)



vertex_shader = """
# version 330
layout(location = 0) in vec3 a_position;
layout(location = 1) in vec4 a_color;

uniform mat4 model;
uniform mat4 projection;
uniform mat4 view;

out vec4 v_color;
void main()
{
    gl_Position = projection * view * model * vec4(a_position, 1.0);
    v_color = a_color;
}
"""

fragment_shader = """
# version 330
in vec4 v_color;

out vec4 out_color;
void main()
{
    out_color = v_color;
}
"""



import pathlib
import craftslicer
from setuptools import setup, find_packages

directory = pathlib.Path(__file__).parent
readme = (directory / "README.md").read_text()
requirements = [i.replace("\n", "") for i in open("requirements.txt", "r").readlines()]

setup(
    name="CraftSlicer",
    version=craftslicer.__version__,
    description="A portal from reality to Minecraft",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/ThatAquarel/CraftSlicer",
    author="Aquarel",
    author_email="flyingrobot910@gmail.com",
    license="MIT",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Win32 (MS Windows)",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "Topic :: Games/Entertainment"
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    entry_points={
        "console_scripts": [
            "craftslicer=craftslicer.__main__:main",
        ]
    }
)



import sys

import qdarkstyle
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *

from craftslicer.core.gl.gl_widget import GlWidget
from craftslicer.core.qt.qt_widgets import PropertiesEdit, ExpandConstraint, SeparatorLine, RunWidget
# noinspection PyUnresolvedReferences
from craftslicer.core.res import qrc_resources


class MainWindow(QMainWindow):
    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        self.setMinimumSize(1280, 720)

        self._create_actions()
        self._create_menu_bar()
        self._create_toolbars()

        self._create_right_layout()
        self.model_canvas = GlWidget(self)
        self.model_canvas.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
        self._create_left_layout()

        grid_layout = QGridLayout()
        grid_layout.addWidget(self.tree, 0, 0)
        grid_layout.addWidget(self.properties_tabs, 1, 0)
        grid_layout.setRowStretch(0, 1)
        grid_layout.setRowStretch(1, 1)

        parent_layout = QGridLayout()
        parent_layout.addLayout(self.model_tab_layout, 0, 0)
        parent_layout.addLayout(grid_layout, 0, 1)
        parent_layout.setColumnStretch(0, 4)
        parent_layout.setColumnStretch(1, 1)

        widget = QWidget()
        widget.setLayout(parent_layout)
        self.setCentralWidget(widget)

        self.show()

    def _create_left_layout(self):
        self.model_tab_layout = QHBoxLayout()
        self.model_tab_layout.addWidget(self.model_canvas)
        self.model_tab_layout.addWidget(self.edit_toolbar)
        self.edit_toolbar.show()
        self.edit_toolbar.setOrientation(Qt.Vertical)

    def _create_right_layout(self):
        self.tree = QTreeWidget()
        self.tree.setColumnCount(1)
        self.tree.setHeaderLabels(["Scene"])

        self.properties_tabs = QTabWidget()
        self.properties_tabs.setTabPosition(QTabWidget.TabPosition.South)
        self.obj_properties_tab = QWidget()
        self.obj_properties_tab_layout = QVBoxLayout()

        self.property_label = QLabel("Properties")
        self.obj_properties_tab_layout.addWidget(self.property_label)
        self.obj_properties_tab_layout.addWidget(SeparatorLine())

        self.obj_location_edit = PropertiesEdit("Location", button_step=1.)
        self.obj_properties_tab_layout.addWidget(self.obj_location_edit)
        self.obj_properties_tab_layout.addWidget(SeparatorLine())

        self.obj_rotation_edit = PropertiesEdit("Rotation")
        self.obj_properties_tab_layout.addWidget(self.obj_rotation_edit)
        self.obj_properties_tab_layout.addWidget(SeparatorLine())

        self.obj_scale_edit = PropertiesEdit("Scale", default_value=1)
        self.obj_properties_tab_layout.addWidget(self.obj_scale_edit)
        self.obj_properties_tab_layout.addWidget(ExpandConstraint())
        self.obj_properties_tab.setLayout(self.obj_properties_tab_layout)

        self.mat_properties_tab = QWidget()

        self.properties_tabs.addTab(self.obj_properties_tab, "Object Properties")
        self.properties_tabs.addTab(self.mat_properties_tab, "Material Properties")

    def _create_actions(self):
        self.new_action = QAction(QIcon(":new.svg"), "&New", self)
        self.open_action = QAction(QIcon(":open.svg"), "&Open", self)
        self.save_action = QAction(QIcon(":save.svg"), "&Save", self)
        self.exit_action = QAction(QIcon(":exit.svg"), "&Exit", self)

        self.select_action = QAction(QIcon(":select.svg"), "&Select", self)
        self.move_action = QAction(QIcon(":move.svg"), "&Move", self)
        self.rotate_action = QAction(QIcon(":rotate.svg"), "&Rotate", self)
        self.scale_action = QAction(QIcon(":scale.svg"), "&Scale", self)
        self.mirror_action = QAction(QIcon(":mirror.svg"), "&Mirror", self)

        self.pan_action = QAction(QIcon(":pan.svg"), "&Pan", self)
        self.view_action = QAction(QIcon(":view.svg"), "&Rotate", self)
        self.zoom_action = QAction(QIcon(":zoom.svg"), "&Zoom", self)

        self.about_action = QAction(QIcon(":about.svg"), "&About", self)

    def _create_menu_bar(self):
        menu_bar = self.menuBar()

        file_menu = menu_bar.addMenu("&File")
        file_menu.addAction(self.new_action)
        file_menu.addAction(self.open_action)
        file_menu.addAction(self.save_action)
        file_menu.addSeparator()
        file_menu.addAction(self.exit_action)

        edit_menu = menu_bar.addMenu("&Edit")
        edit_menu.addAction(self.pan_action)
        edit_menu.addAction(self.view_action)
        edit_menu.addAction(self.zoom_action)
        edit_menu.addSeparator()
        edit_menu.addAction(self.select_action)
        edit_menu.addAction(self.move_action)
        edit_menu.addAction(self.rotate_action)
        edit_menu.addAction(self.scale_action)
        edit_menu.addAction(self.mirror_action)

        help_menu = menu_bar.addMenu("&Help")
        help_menu.addAction(self.about_action)

        self.run_widget = RunWidget()
        menu_bar.setCornerWidget(self.run_widget, Qt.TopRightCorner)

    def _create_toolbars(self):
        self.edit_toolbar = self.addToolBar("Edit")
        self.removeToolBar(self.edit_toolbar)

        self.edit_toolbar.setMovable(False)
        self.edit_toolbar.addAction(self.pan_action)
        self.edit_toolbar.addAction(self.view_action)
        self.edit_toolbar.addAction(self.zoom_action)
        self.edit_toolbar.addSeparator()
        self.edit_toolbar.addAction(self.select_action)
        self.edit_toolbar.addAction(self.move_action)
        self.edit_toolbar.addAction(self.rotate_action)
        self.edit_toolbar.addAction(self.scale_action)
        self.edit_toolbar.addAction(self.mirror_action)


def main():
    app = QApplication(sys.argv)
    app.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt5'))

    MainWindow()
    app.exec_()


if __name__ == '__main__':
    main()



import socket
import struct

HOST = "localhost"
PORT = 25568


def send(path: str):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect((HOST, PORT))

    sock.sendall(struct.pack(">hi", 2, len(path)))
    sock.sendall(path.encode("UTF-8"))

    data = struct.unpack(">hi", sock.recv(6))
    sock.close()

    return data
